#ifndef ACOUSTIC_SOLVER_ALGORITHM_HPP
#define ACOUSTIC_SOLVER_ALGORITHM_HPP

#include <language/utils/FunctionSymbolId.hpp>
#include <language/utils/InterpolateItemValue.hpp>
#include <output/VTKWriter.hpp>
#include <scheme/AcousticSolver.hpp>
#include <scheme/IBoundaryConditionDescriptor.hpp>
#include <scheme/IBoundaryDescriptor.hpp>
#include <scheme/PressureBoundaryConditionDescriptor.hpp>
#include <scheme/SymmetryBoundaryConditionDescriptor.hpp>
#include <scheme/VelocityBoundaryConditionDescriptor.hpp>

template <size_t Dimension>
struct AcousticSolverAlgorithm
{
  using ConnectivityType = Connectivity<Dimension>;
  using MeshType         = Mesh<ConnectivityType>;
  using MeshDataType     = MeshData<Dimension>;
  using UnknownsType     = FiniteVolumesEulerUnknowns<MeshType>;

  AcousticSolverAlgorithm(const AcousticSolverType& solver_type,
                          std::shared_ptr<const IMesh> i_mesh,
                          const std::vector<std::shared_ptr<const IBoundaryConditionDescriptor>>& bc_descriptor_list,
                          const FunctionSymbolId& rho_id,
                          const FunctionSymbolId& u_id,
                          const FunctionSymbolId& p_id);
};

#endif   // ACOUSTIC_SOLVER_ALGORITHM_HPP
